/*
 * @(#)TertrisP.h
 *
 * Copyright 1993 - 2008  David A. Bagley, bagleyd@tux.org
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995  Q. Alex Zhao, azhao@cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Private header file for Tertris */

#ifndef _TertrisP_h
#define _TertrisP_h

#include "xwin.h"
#include "timer.h"
#include "polyiamond.h"
#include "Tertris.h"

#ifdef WINVER
extern void destroyGame(HBRUSH brush);
extern void resizeGame(TertrisWidget w);
extern void sizeGame(TertrisWidget w);
extern void initializeGame(TertrisWidget w, HBRUSH brush);
extern void exposeGame(TertrisWidget w);
extern void hideGame(TertrisWidget w);
extern void clearGame(TertrisWidget w);
extern void enterGame(TertrisWidget w);
extern void leaveGame(TertrisWidget w);
extern void moveGameInput(TertrisWidget w, int direction);
extern void soundGame(TertrisWidget w);
extern Boolean incrementGameLevel(TertrisWidget w);
extern Boolean decrementGameLevel(TertrisWidget w);
extern Boolean incrementGameUnits(TertrisWidget w);
extern Boolean decrementGameUnits(TertrisWidget w);
extern Boolean incrementGameColumns(TertrisWidget w);
extern Boolean decrementGameColumns(TertrisWidget w);
extern void fixedfillGame(TertrisWidget w);
extern void suddenAppearGame(TertrisWidget w);
extern Boolean diagonalGame(TertrisWidget w);
extern Boolean mixedGame(TertrisWidget w);
extern void bonusGame(TertrisWidget w);
extern void cwGame(TertrisWidget w);
extern void plainPieceGame(TertrisWidget w);
extern void falldownGame(TertrisWidget w);
extern void pauseGame(TertrisWidget w);
extern void showNextGame(TertrisWidget w);
extern void gridGame(TertrisWidget w);
extern void fillGame(TertrisWidget w, int n);
extern void setGame(TertrisWidget w, int reason);
extern void setGameNext(TertrisWidget w, int reason,
	int randomNumber, int randomRotation, int randomReflection,
	int units, Boolean diagonal, Boolean mixed, Boolean newBonus);

#else

/* This gets around C's inability to do inheritance */
typedef struct _TertrisClassPart {
	int         ignore;
} TertrisClassPart;

typedef struct _TertrisClassRec {
	CoreClassPart coreClass;
	TertrisClassPart tertrisClass;
} TertrisClassRec;

extern TertrisClassRec tertrisClassRec;
#endif

#ifndef BUMPSOUND
#define BUMPSOUND "bump"
#endif

#define SYMBOL ':'

#define NUM_FLASHES 4

#define MAX_SIDES 6
#define MAX_ORIENT 2
#define MIN_LEVELS 1
#define MAX_LEVELS 20
#define FG_SHADES 3
#define BG_SHADES 5

#define THRESHOLD(x)	((x)*10)

/*** types ***/

typedef enum {FALL, DROP, MOVE_LEFT, MOVE_RIGHT, ROTATE, REFLECT} move_t;

typedef struct {
	int         units;
	Boolean     diagonal, mixed, bonus;
} Spec;

typedef struct {
	int         units, polyNumber;
	int         xpos, ypos;
	int         size, colorNumber, direction;
	int         bonus, randomRotation, randomReflection;
	long        randomNumber;
} Thing;

typedef struct {
	int         pmid, cid;
} Field;

typedef struct _TertrisPart {
	Thing       curThing, nextThing;
	Field      *field;
	GC          thingGC[MAX_START_POLYIAMONDS][FG_SHADES];
	Boolean     fixedFill, suddenAppear, cw, plainPiece, showNext, grid;
	Boolean     invert, sound, resetTime, firstFall, paused, gotNew;
	int         level, fill, completedRows;
	int         startLevel, score, newRows;
	Spec        current, next;
	int         units, columns, rows;
	Boolean     diagonal, mixed, bonus;
	int         delay, numberPolyiamonds, pauseStart;
	int         currentPosition, dropPaused;
	Boolean     fall, started, quit, act;
	Boolean     mono, reverse, focus;
	Boolean     scoreOnly, versionOnly;
	int         triWidth, triHeight;
	Position    viewWidth, viewHeight;
	Point       offset;
	GC          xorGC;
	GC          inverseGC[BG_SHADES];
#ifdef WINVER
	char        userName[81], scoreFile[81], bumpSound[81];
#else
	int         menu, pixmapSize;
	char       *userName, *scoreFile, *bumpSound;
	Pixel       foreground, background;
	Pixel       thingColor[MAX_START_POLYIAMONDS];
	XtCallbackList select;
#endif
} TertrisPart;

typedef struct _TertrisRec {
	CorePart    core;
	TertrisPart  tertris;
} TertrisRec;

#ifdef WINVER
extern int brighter(const int light);
extern int darker(const int light);
#else
extern Pixel brighter(TertrisWidget w, Pixel pixel);
extern Pixel darker(TertrisWidget w, Pixel pixel);
#endif
extern void fillPolygon(TertrisWidget w, int pm, int color, Boolean invert,
	Point *points, Boolean graphicalXor);
extern void redoNext(TertrisWidget w);
#endif /* _TertrisP_h */
